/*
 * bfdgetchar() : buffered read(), returns 1 char
 * @f: the FILE* to read from
 * @c: if this is non-0 it acts like ungetc
 * */

#include <stdio.h>
#include <unistd.h>

#include "io.h"

/*
 * buffered getc()
 * @f: stream to read from
 *
 * returns read char
 * */

char
bfgetc (FILE *f) {
	return bfmodc (f, 0);
}

/*
 * buffered ungetc()
 * @f: stream to push onto
 * @c: char to push onto buffered stream
 *
 * */

void
bfungetc (FILE *f, char c) {
	bfmodc (f, c);
	return;
}

/*
 * act like buffered getc() and ungetc()
 * @f: the stream to modify
 * @c: if this is non-0 acts like ungetc and pushes c onto buffer or onto f 
 * 
 * you shouldn't use this function directly, rather use bfgetc() and bfungetc()
 * 
 * returns character or 0
 * */

char
bfmodc (FILE *f, char c) {
	static char buf[BUFSIZ];
	static char *bufp = buf;

	if (c != 0) {
		if (bufp != buf) {
			bufp--;
			return 0;
		}
		else {
			bfungetc (f, c);
			return 0;
		}
	}

	if (*bufp != 0) {
		return *bufp++;
	}
	else {
		/* buffer is empty */
		bufp = buf;
		if (fgets (buf, sizeof buf, f) != NULL)
			return *bufp++;
		else
			return EOF;
	}
}

char *
getword (FILE *f, char *s)
{
	extern int lineno;
	char c;
	int i=0;

	/* skip whitespace */
	while ((c = bfgetc (f)) == ' ' || c == '\t' || c == '\n' || c == '\r') {
		if (c == '\n')
			lineno++;
	}

	if (c == '"') {
		while ((c = bfgetc (f)) != '"')
			*s++ = c;
	}
	else {
		while (c != EOF && c != ' ' && c != '\t' &&
			     c != '\n' && c != '\r' && c != ';' && c != '}' && c != '{') {
			*s++ = c;
			i++;
			c = bfgetc (f);
		}

		if (c == ';' || c == '}' || c == '{') {
			if (i > 0) {
				bfungetc (f, c);  /* push back read character */
			}
			else {
				*s++ = c;
			}
		}
	}

	*s = '\0';

	if (c == '\n')
		lineno++;

	if (c == EOF)
		return NULL;
	else
		return s;
}

void
to_next_line (FILE *f)
{
	while (bfgetc (f) != '\n')
		;
}
